#!/usr/bin/env bash

usage="Usage: $0 -l <layer_id> -k <API_key> -d <destination_dir> [-f <format>]"

if [[ $# -lt 6 ]]
then
    echo "$usage" >&2
    exit 1
fi

while getopts ":l:k:d:f:" opt
do
    case $opt in
        l)
            number_regex='^[0-9]+$'
            if ! [[ "$OPTARG" =~ $number_regex ]]
            then
                echo "Layer ID must be a number" >&2
                echo "$usage" >&2
                exit 1
            fi
            layer_id="$OPTARG"
            ;;
        k)
            key_regex='^[0-9a-f]+$'
            if ! [[ "$OPTARG" =~ $key_regex ]]
            then
                echo "Invalid API key format" >&2
                echo "$usage" >&2
                exit 1
            fi
            api_key="$OPTARG"
            ;;
        d)
            if ! [[ -d "$OPTARG" ]]
            then
                echo "Destination directory $4 does not exist"
                echo "$usage" >&2
                exit 1
            fi
            dest_dir="$OPTARG"
            ;;
        f)
            case "$OPTARG" in
                "esri shapefile"|"mapinfo file"|"csv"|"dxf"|"kml")
                    format="$OPTARG"
                    ;;
                *)
                    echo "Invalid format. Format must be one of: esri shapefile, mapinfo file, csv, dxf, kml" >&2
                    echo "$usage" >&2
                    exit 1
                    ;;
            esac
            ;;
        \?)
            echo "Invalid option -$OPTARG" >&2
            echo "$usage" >&2
            exit 1
            ;;
    esac
done

# default format is shp
if [[ -z "$format" ]]
then
    format="esri shapefile"
fi

if [[ -z "$layer_id" ]] || [[ -z "$api_key" ]] || [[ -z "$dest_dir" ]]
then
    echo "Missing -l, -k and/or -d arguments"
    echo "$usage" >&2
    exit 1
fi

export_attempts=5;
retry_timer=15
base_url="https://editor.giscloud.com/datasource/export?task=exportlayer&module=layer&name=layer_export"


temp_file="${dest_dir}/layer${layer_id}.tmp"

url="${base_url}&layerid=${layer_id}&format=${format}&api_key=${api_key}"

export_successful=false
n=0
until [[ "$n" -ge "$export_attempts" ]]
do
    n=$((n+1))

    if ! [[ "$n" -eq 1 ]]
    then
        echo "Attempt #$((n-1)) failed. Retrying in $retry_timer seconds. Attempts left: $((export_attempts-n+1))"
        sleep "$retry_timer"
    fi

    echo "Trying to request layer data export from URL: $url"

    if ! wget -q --output-document "${temp_file}" "$url";
    then
        echo "Export request failed, url: $url"
        continue
    fi

    if ! [[ -f "$temp_file" ]]
    then
        echo "Temporary output file ($temp_file) does not exist"
        continue
    fi

    response_body=$(<"$temp_file")

    if [[ -z "$response_body" ]]
    then
        echo "Export request response body is empty"
        continue
    fi

    if [[ "$response_body" == *'"result": "no"'* ]]
    then
        echo "Operation unsuccessful: $response_body"
        continue
    fi

    if [[ "$response_body" == *'"result": "yes"'* ]]
    then
        echo "Invalid response: $response_body"
        continue
    fi

    export_url=$(sed 's/.*"url":"//g' "$temp_file" | sed 's/".*//g' | sed 's/\\\//\//g')

    if [[ -z "$export_url" ]]
    then
        echo "Invalid response - can't find URL: $response_body"
        continue
    fi

    echo "$export_url"

    export_filename="${dest_dir}${export_url/*\///}"
    export_filename=$(echo -n "$export_filename" | sed "s/.zip$/_$(date +%s).zip/")

    echo "Trying to fetch export file from URL: ${export_url}, and write it to ${export_filename}"

    if ! wget -q --output-document "$export_filename" "$export_url";
    then
        echo "Error retrieving exported file from url: $export_url"
        continue
    fi

    echo "Layer successfully exported to file $export_filename"

    rm "$temp_file"

    export_successful=true

    break
done

if [[ "$export_successful" == false ]]
then
    echo "Fetching the export failed $export_attempts times. Giving up"
    exit 1
fi

exit 0

